
// LCD drivers
// PORT A is the control port
// PORT C is the data port

#define Draw 0
#define Undraw 1
#define Toggle 2

// LCD control definitions
#define LcdReset 0x00 // LCD reset
#define DataWrite 0x03 // Data write
#define DataSetup 0x07 // Data setup
#define CommandWrite 0x43 // Command write
#define DataRead 0x45 // Data receive
#define LcdNop 0x47 // Do nothing
#define CommandSetup 0x47 // Command setup

// LCD command definitions
#define SystemSet 0x40 // Initialize LCD
#define MemoryWrite 0x42 // Write to display memory
#define MemoryRead 0x43 // Read display memory
#define Scroll 0x44 // Set start address, regions
#define SetCursor 0x46 // Set cursor address
#define ReadCursor 0x47 // Read cursor address
#define CursorRight 0x4C // Cursor right
#define CursorLeft 0x4D // Cursor left
#define CursorUp 0x4E // Cursor up
#define CursorDown 0x4F // Cursor down
#define Standby 0x53 // Go to standby
#define DisplayOff 0x58 // Disable display
#define DisplayOn 0x59 // Enable display
#define HorizontalScroll 0x5A // Set horizontal scroll position
#define Overlay 0x5B // Set display overlay
#define CharAddress 0x5C // Start address of character RAM
#define CursorForm 0x5D // Set cursor type

#include <math.h>

flash unsigned char SystemCommand[8]={0x30,0x87,0x07,0x27,0x2F,0xC7,0x28,0x00};
flash unsigned char ScrollCommand[6]={0x00,0x00,0xC8,0xE8,0x03,0xC8};

void WriteCommand(unsigned char Command); // Write command to LCD
void WriteData(unsigned char Data); // Write data to LCD
void MoveCursor(unsigned int Address); // Move the cursor
void InitLCD(void); // Initialize the LCD
void ClearLCD(void); // Clear the LCD
void DrawIntro(void); // Draw the game introduction
void PrintBuffer(void); // Print contents in LCD buffer string      
// reads a byte from lcd memory, returns that
unsigned char ReadData();
// draw a point on the lcd
void lcd_pt(int x, int y, char action);
// draw a line between two points on the lcd
void lcd_line(int x1, int y1, int x2, int y2, char action);


// LCD functions
void WriteCommand(unsigned char Command) // Write command sequence to LCD
     begin
PORTC = Command;
PORTA = CommandSetup;
PORTA = CommandWrite;
PORTA = CommandSetup;
end

void WriteData(unsigned char Data) // Write data sequence to LCD
     begin
PORTC = Data;
PORTA = DataSetup;
PORTA = DataWrite;
PORTA = DataSetup;
end

void MoveCursor(unsigned int Address) // Move the cursor
     begin
WriteCommand(SetCursor);
WriteData(Address);
WriteData(Address>>8);
end

void InitLCD(void) // Initialization routine (see datasheet, Section 9)
     begin
unsigned char i;

DDRA = 0xff; // Set control port to output
DDRC = 0xff; // Set data port to output
PORTA = LcdNop;
delay_ms(5); // Delay 2 ms
PORTA = LcdReset; // Assert reset
delay_ms(3);
PORTA = LcdNop;
delay_ms(2);

WriteCommand(SystemSet);
for(i=0;i<8;i++)
     begin
WriteData(SystemCommand[i]); // System set commands
end

WriteCommand(Scroll); // Scrolling, addressing
for(i=0;i<6;i++)
     begin
WriteData(ScrollCommand[i]); // Scrolling commands
end

WriteCommand(HorizontalScroll);
WriteData(0x00); // No scrolling

WriteCommand(Overlay); // Overlay
WriteData(0x01); // Set to 2-layer text/graphics, XOR composition
WriteCommand(DisplayOff);
WriteData(0x14); // No cursor
ClearLCD(); // Clear the LCD (blank character and graphic spaces)

MoveCursor(0x0000);
WriteCommand(CursorForm); // Choose cursor form
WriteData(0x04); // Cursor horizontal
WriteData(0x86); // Cursor vertical, block vertical

WriteCommand(DisplayOn); // Turn on the display
end

void ClearLCD(void) // LCD clearing function
     begin
unsigned int i;

WriteCommand(CursorRight);
MoveCursor(0x0000);
WriteCommand(MemoryWrite);

for(i=0;i<1000;i++)
     begin
WriteData(0x20); // Clear the character space
end

MoveCursor(0x03E8);
WriteCommand(MemoryWrite);

for(i=0;i<8000;i++)
     begin
WriteData(0x00); // Clear graphics space
end
end


void PrintBuffer(void) // Print the LCD buffer
     begin
unsigned char i = 0;

WriteCommand(MemoryWrite);
while(lcd_buffer[i] != 0)
     begin
WriteData(lcd_buffer[i++]);
end
end


//reads a byte from lcd memory, returns that
unsigned char ReadData()
{
  unsigned char read_data;
  WriteCommand(MemoryRead);
  DDRC = 0x00;
  PORTC = 0x00;
  PORTA = DataSetup;
  PORTA = DataRead;
  PORTA = DataSetup;
  read_data = PINC;
  DDRC = 0xFF;
  PORTC = LcdNop;
  return read_data;
}




void lcd_pt(int x, int y, char action) {
  // x is the x coordinate
  // y is the y coordinate
  // action is 0 for erase, 1 for write, 2 for toggle

  unsigned int address; //address of the byte containing the pixel to be written
  char bit_to_write = 0;
  char old_byte;
  char byte_to_write;
  int i;

// address = (x>>3) + (y*40)
  address = (x>>3) + ((y<<3) * 5) + 1000;


  MoveCursor(address);
  old_byte = ReadData();
  MoveCursor(address);

  switch (action) {
  case 0:
    bit_to_write = 1 << (7 -(x & 0x7));
    bit_to_write = 0xFF ^ bit_to_write;
    byte_to_write = old_byte & bit_to_write;
    break;
  case 1:
    bit_to_write = 1 << (7 -(x & 0x7));
    byte_to_write = old_byte | bit_to_write;
    break;
  case 2:
    bit_to_write = 1 << (7 - (x & 7));
    byte_to_write = old_byte ^ bit_to_write;
    break;
  }

  // MoveCursor(address);
  WriteCommand(MemoryWrite);
  WriteData(byte_to_write);

}

//Adapted from Lab 4 code to draw lines on the TV screen (Bruce Land)
//==================================
//plot a line
//at x1,y1 to x2,y2 with color 1=white 0=black 2=invert
//NOTE: this function requires signed chars
//Code is from David Rodgers,
//"Procedural Elements of Computer Graphics",1985
void lcd_line(int x1, int y1, int x2, int y2, char action)
{
  int e;
  signed int dx,dy,j, temp;
  signed int s1,s2, xchange;
  signed int x,y;

  x = x1;
  y = y1;
  dx = abs(x2-x1);
  dy = abs(y2-y1);
  s1 = sign(x2-x1);
  s2 = sign(y2-y1);
  xchange = 0;
  if (dy>dx)
    {
      temp = dx;
      dx = dy;
      dy = temp;
      xchange = 1;
    }
  e = ((int)dy<<1) - dx;
  for (j=0; j<=dx; j++)
    {
      lcd_pt(x, y, action) ;
      if (e>=0)
	{
	  if (xchange==1) x = x + s1;
	  else y = y + s2;
	  e = e - ((int)dx<<1);
	}
      if (xchange==1) y = y + s2;
      else x = x + s1;
      e = e + ((int)dy<<1);
    }
}
